<?php
/**
 * @package Forums by Website Toolbox
 * @author Website Toolbox
 */

/**
 * Plugin Name: Forums by Website Toolbox
 * Plugin URI:  https://www.websitetoolbox.com/wordpress
 * Description: Website Toolbox is the easiest way to add a forum to WordPress. The forum plugin features single sign on and seamlessly embeds into your website.
 * Author:      Website Toolbox
 * Author URI:  https://www.websitetoolbox.com/wordpress
 * Version:     1.3.9
 */
include("wt_forum_include.php");
include("forumHook.php");
include("admin/admin.php");

/* Purpose: Set page content on the front end according to the basic theme.
Parameter: None
Return: None */
function websitetoolbox_lol($content) {
	$websitetoolboxpage_id = get_option('websitetoolbox_pageid');
	$page_content = get_page($websitetoolboxpage_id);
	$page_content = $page_content->post_content;
	$theme_data = wp_get_theme();
	$wrap_pre = "<style>.nocomments { display: block; }</style>";
	$wrap_post = "";
	if($theme_data['Name']=="WordPress Default" && strpos($theme_data['Description'], '>Kubrick<')==90) {
		$wrap_pre .= "<div style='background-color: white;'>";
		$wrap_post .= "</div>";
	}
	if($theme_data['Template'] == "twentyeleven") {
		$wrap_pre .= <<<STYLE
		<style type="text/css">
		.singular .entry-header, .singular .entry-content, .singular footer.entry-meta, .singular #comments-title {
		width: 100%;
		}
		.singular #content, .left-sidebar.singular #content {
		margin: 0 1.5%;
		}
		.page-id-$websitetoolboxpage_id  .entry-title {display: none;}

		#main { padding: 0; }
		.singular.page .hentry { padding: 0; }
		</style>
STYLE;
	}
	return <<<EMBED
	$wrap_pre
	$page_content
	$wrap_post
EMBED;
}

/* Purpose: create a new page for front end.
Parameter: None
Return: None */
function websitetoolbox_init() {
    $websitetoolboxpage_id = get_option('websitetoolbox_pageid');
	if(is_page($websitetoolboxpage_id)) {
        // Get post data on the basis of postid
        $postData = get_post($websitetoolboxpage_id);
        if($postData && $postData->post_status!='publish') {
            $postData->post_status = 'publish';
            wp_update_post($postData);
        }
        add_filter("the_content", "websitetoolbox_lol");
    }
}

/* Purpose: sent HTTP request for get authtoken if user try to login on the WordPress website.
Parameter: None
Return: None */
function wt_login_user($user_login) {
	$user_obj = new WP_User(0,$user_login);
	$username = $user_obj->user_login;
	$email 	  = $user_obj->user_email;

	$forum_api		= get_option("websitetoolbox_api");
	$forum_url		= get_option("websitetoolbox_url");

	// create URL to get authentication token.
	$URL = $forum_url."/register/setauthtoken";
	// Append fileds email and password to create an account on the related forum if account not exist.
	$fields = array('type' =>'json', 'apikey' => $forum_api, 'user' => $username, 'email' => $email);
	// Get plain password from WordPress login form value and append it with HTTP request.
	if(isset($_POST['pwd']) && $_POST['pwd'] != "") {
		$fields['pw'] = $_POST['pwd'];
	}
	// Send http or https request to get authentication token.
	$response_array = wp_remote_post($URL, array('method' => 'POST', 'body' => $fields));
	//Check if http/https request could not return any error then filter JSON from response
	if(!is_wp_error( $response_array )) {
		$response = trim(wp_remote_retrieve_body($response_array));
		$response = json_decode($response);
		if(!$response->{'message'}) {
			$authtoken = $response->{'authtoken'};
			// Check if "-" exist in the authtoken then first part is authtoken (login on the forum) second part is access token login on Chat Room.
			if( strpos( $authtoken, "-" ) !== false ) {
				$tokens = explode("-",$authtoken);
				$authtoken = $tokens[0];
				$access_token = $tokens[1];
				$chatroom_url = $response->{'chatroom_url'};
			}
			if(!empty($_POST['rememberme'])) {
				setcookie('wt_login_remember', "checked", time() + 1296000, COOKIEPATH, COOKIE_DOMAIN);
			}
			// Save authentication token into cookie for one day to use into SSO logout.
			setcookie('wt_logout_token', $authtoken, time() + 86400, COOKIEPATH, COOKIE_DOMAIN);
			// Save access token into cookie for one day to use into SSO logout from Chat Room.
			if($access_token) {
				setcookie('wtchat_logout_token', $access_token, time() + 86400, COOKIEPATH, COOKIE_DOMAIN);
				setcookie('wtchat_url', $chatroom_url, time() + 86400, COOKIEPATH, COOKIE_DOMAIN);
			}
			#Save authentication token into session variable.
			save_authtoken($authtoken,$access_token,$chatroom_url);
			return true;
		}
	}
}

/* Purpose: This function is used to set authentication token into session variable if user logged-in.
Param: authentication token
Return: Nothing */
function save_authtoken($authtoken,$access_token,$chatroom_url) {
	$_SESSION['wtb_login_auth_token'] = $authtoken;
	// Save access token into a session to login user on the caht room
	if($access_token) {
		$_SESSION['wtbchat_login_auth_token'] = $access_token;
		$_SESSION['wtbchat_url'] = $chatroom_url;
	}
}

/* Purpose: execute HTTP request to create an account on the related forum once user create account on the WordPress website.
Parameter: None
Return: None */
function wt_register_user($userid) {

	$user_obj = new WP_User($userid);
	$forum_api		= get_option("websitetoolbox_api");
	$forum_url		= get_option("websitetoolbox_url");

	$login_id = $user_obj->ID;
	$login 	  = $user_obj->user_login; # ie: JohnD223

	// pass1 used for password filed (WordPress Registration form) in older version (below 4.3).
	// pass1-text used for password filed in WordPress versions (4.3 and above).
	// Pass this password into SSO regisration request if not peresent then sent blank password.
	if (isset($_POST['pass1']) && $_POST['pass1'] != "") {
		$password = $_POST['pass1'];
	} elseif (isset($_POST['pass1-text']) && $_POST['pass1-text'] != "") {
		$password = $_POST['pass1-text'];
	} else {
		$password = '';
	}

	$email 	  = $user_obj->user_email;
	$display_name = $user_obj->display_name; # ie: John Doe
	$first_name = $user_obj->first_name; # ie: John
	$last_name = $user_obj->last_name; # ie: Doe
	$fullname = $first_name." ".$last_name;

	// URL to create a new account on forum.
	$URL = $forum_url."/register/create_account";
	// Fields array.
	$fields = array('apikey' => $forum_api, 'member' => $login, 'pw' => $password, 'email' => $email, 'name' => $fullname);
	// Sent https/https request on related forum to create an account on the related forum.
	$response_array = wp_remote_post($URL, array('method' => 'POST', 'body' => $fields));
	return true;
}

/* Purpose: If a user Logged-in/logged-out on WordPress site from front end/admin section write an image tag after page load to loggout from forum.
Parameter: None
Return: None */
function ssoLoginLogout() {
	// If user logged-out from the Forum then call wp_logout function to logged-out from WordPress site as well as forum.
	if(isset($_GET['action']) && $_GET['action']=='ssoLogout' && is_user_logged_in()) {
		unset($_SESSION['wtb_account_settings']);
		wp_logout();
		exit;
	}

	// If user logged-out from WordPress Site
	if (isset($_SESSION['wtb_login_auth_token'])) {
		$login_auth_url = get_option('websitetoolbox_url');
		// Remove http: | https: from the Forum URL so that it will not print into IMG tag src attribbute.
		$login_auth_url = preg_replace('#^https?:#i', '', $login_auth_url);
		$login_auth_url = $login_auth_url."/register/dologin?authtoken=".$_SESSION['wtb_login_auth_token'];
		if(isset($_COOKIE['wt_login_remember'])) {
			$login_auth_url = $login_auth_url."&remember=".$_COOKIE['wt_login_remember'];
		}

		/* Print image tag on the login landing success page to sent login request on the related forum */
		echo '<img src="'.$login_auth_url.'" border="0" width="0" height="0" alt="">';
		/* Print image tag on the login landing success page to sent login request on the related Chat Room */
		if(isset($_SESSION['wtbchat_login_auth_token'])) {
			// Remove http: | https: from the Chat Room URL so that it will not print into IMG tag src attribbute.
			$chatlogin_auth_url = $_SESSION['wtbchat_url'];
			$chatlogin_auth_url = preg_replace('#^https?:#i', '', $chatlogin_auth_url);
			$chatlogin_auth_url = $chatlogin_auth_url."/sso/token/login?access_token=".$_SESSION['wtbchat_login_auth_token'];
			if(isset($_COOKIE['wt_login_remember'])) {
				$chatlogin_auth_url = $chatlogin_auth_url."&rememberMe=1";
			}
			echo '<img src="'.$chatlogin_auth_url.'" border="0" width="0" height="0" alt="">';
		}
		unsetLoginSession();
		return false;
	}
	if(!is_user_logged_in() && isset($_COOKIE['wt_logout_token'])) {
		$logout_auth_url = get_option('websitetoolbox_url')."/register/logout?authtoken=".$_COOKIE['wt_logout_token'];
		$logout_auth_url = preg_replace('#^https?:#i', '', $logout_auth_url);
		/* Print image tag on the header section sent logout request on the related forum */
		echo '<img src="'.$logout_auth_url.'" border="0" width="0" height="0" alt="">';
		/* If user logged-in on the chat room then logged-out from the chat room */
		if(isset($_COOKIE['wtchat_logout_token'])) {
			$chatlogout_auth_url = $_COOKIE['wtchat_url']."/sso/token/logout?access_token=".$_COOKIE['wtchat_logout_token'];
			echo '<img src="'.$chatlogout_auth_url.'" border="0" width="0" height="0" alt="">';
		}

		resetCookieOnLogout();
		return false;
	}

	// User logged-out from wordpress site and session varible exist then unset the session varible.
	if(!is_user_logged_in() && isset($_SESSION['wtb_account_settings'])) {
		unset($_SESSION['wtb_account_settings']);
	}
}

/* Purpose: This function is used to unset session variable if user logged-in so that this session variable did't effect any other place.
Param1: None
Return: None */
function unsetLoginSession() {
	unset($_SESSION['wtb_login_auth_token']);
	unset($_SESSION['wtbchat_login_auth_token']);
	unset($_SESSION['wtbchat_url']);
}

/* Purpose: This function is used to reset cookie variable if user logged-out from the WordPress website so that authtoken did't effected.
Param1: type (login/logout)
Return: None */
function resetCookieOnLogout() {
	setcookie("wt_logout_token", '', 0);
	setcookie("wtchat_logout_token", '', 0);
	setcookie("wtchat_url", '', 0);
}

/* Purpose: sent logout request to forum once user logged-out from the WordPress website.
Parameter: none
Return: none */
function wtbForumLogout() {
    global $wp;
    // Preapred logut URL for forum and append redirect URL into it so that user redirected to related page after logout from the forum.
    $forum_url	= get_option("websitetoolbox_url");
    $forum_url .= '/register/logout?redirect=';

    // If redirect_to exist in the REQUEST_URI then get redirected URL.
    // If redirect_to not exist in the URL then get page URL.
    // append it with forum logout URL and redirected it to forum so that after logged-out from the forum redirected to related page.
    if(isset($_REQUEST['redirect_to'])) {
        $redirec_url = $_REQUEST['redirect_to'];
    } else if(get_permalink()) {
        $redirec_url = get_permalink();
    } else {
        $redirec_url = wp_get_referer();
        if(strpos($redirec_url, "action=logout")) {
            $redirec_url = str_replace('action=logout', '', $redirec_url) ;
        }
    }

    $wtbLogoutUrl = $forum_url.urlencode($redirec_url);
    resetCookieOnLogout();
    wp_redirect($wtbLogoutUrl);
    exit;
}

# for URl making
if (!function_exists('esc_attr')) {
	function esc_attr($attr){return attribute_escape( $attr );}
	function esc_url($url){return clean_url( $url );}
}

/* Purpose: set forum page link in WordPress menu in case of forum is not embedded on WordPress website.
Parameter: None
Return: None */
if(get_option("websitetoolbox_redirect") == '') {
	function filter_page_links_wtb ($link, $post) {
		if(isset($post->ID)) {
			$id = $post->ID;
		} else {
			$id = $post;
		}
		#get array
		$newCheck = get_main_array();
		if(!is_array($newCheck)) { $newCheck = array(); }
		#check array according to the key if forum used external url
		if(array_key_exists($id, $newCheck)) {
			$matchedID = $newCheck[$id];
			$newURL = $matchedID['_links_to'];
			if(strpos($newURL,get_option('home'))>=0 || strpos($newURL,'www.')>=0 || strpos($newURL,'http://')>=0 || strpos($newURL,'https://')>=0) {
				if($matchedID['_links_to_target'] == 'websitetoolbox') {
					$newURL = trim($matchedID['_links_to']);
					// Added / at the end of forum url if open into parent window.
					if(!preg_match("/\/$/", $newURL)) {
						$link = $newURL."/";
					}
				} else {
					$link = esc_url( $newURL );
				}
			} else {
				if($matchedID['_links_to_target'] == 'websitetoolbox') {
					// Added / at the end of forum url if open into parent window.
					if(!preg_match("/\/$/", $newURL)) {
						$link = $newURL."/";
					}
				} else {
					$link = esc_url( get_option( 'home').'/'. $newURL );
				}
			}
		}
		return $link;
	}
	add_filter('page_link', 'filter_page_links_wtb', 20, 2);
	add_filter('post_link', 'filter_page_links_wtb', 20, 2);
}

# Get main array from the post meta and post table according to redirest url
function get_main_array(){
	global $wpdb;
	$theArray = array();

	$theqsl = "SELECT * FROM $wpdb->postmeta a, $wpdb->posts b  WHERE a.`post_id`=b.`ID` AND b.`post_status`!='trash' AND (a.`meta_key` = '_wtbredirect_active' || a.`meta_key` = '_links_to' || a.`meta_key` = '_links_to_target' || a.`meta_key` = '_links_to_type') ORDER BY a.`post_id` ASC;";
	$thetemp = $wpdb->get_results($theqsl);
	if(count($thetemp)>0){
		foreach($thetemp as $key){
			$theArray[$key->post_id][$key->meta_key] = $key->meta_value;
		}
		foreach($thetemp as $key){
			// defaults
			if(!isset($theArray[$key->post_id]['_links_to'])){$theArray[$key->post_id]['_links_to']	= 0;}
			if(!isset($theArray[$key->post_id]['_links_to_type'] )){$theArray[$key->post_id]['_links_to_type']				= 302;}
			if(!isset($theArray[$key->post_id]['_links_to_target'])){$theArray[$key->post_id]['_links_to_target']	= 0;}
		}

	}
	return $theArray;
}

/* Purpose: Function is used to append authtoken at the end of URL if user logged-in on wordpress site and then clicks on the forum link.
Parameter: Manual link and post id.
Return: Manual links */
function changeForumLink($items, $post){
    // Append authtoken in the forum link if user logged-in on WordPress site and forum open in window independently.
    if(is_user_logged_in() && $post == get_option('websitetoolbox_pageid') && get_option("websitetoolbox_redirect") == '') {
        $add_token = appendTokenInURL();
        $items = add_query_arg( $add_token, $items );
    }
    return $items;
}
add_filter('page_link', 'changeForumLink', 20, 2);
add_filter('post_link', 'changeForumLink', 20, 2);

/* Purpose: Function is used to append authentication token into the forum URL.
Parameter: page content.
Return: replace page content */
function updatePageContent($content) {
    // Append authtoken in the embed code if user logged-in on WordPress site and forum open in iframe.
    if($GLOBALS['post']->ID == get_option('websitetoolbox_pageid') && get_option("websitetoolbox_redirect") != '' && is_user_logged_in()) {
        if(preg_match('#^https?://#', get_option('websitetoolbox_url'))) {
            $wtb_url = get_option('websitetoolbox_url');
        } else {
            $wtb_url = "http://".get_option('websitetoolbox_url');
        }
        $wtb_embed_url = preg_replace('#^https?:#i', '', $wtb_url);
        $auth_token = "?authtoken=".$_COOKIE['wt_logout_token'];
        if(isset($_COOKIE['wtchat_logout_token'])) {
            $auth_token .= "-".$_COOKIE['wtchat_logout_token'];
        }
        if(isset($_COOKIE['wt_login_remember'])) {
            $auth_token .= "&remember=".$_COOKIE['wt_login_remember'];
        }
        $embedURL = $wtb_embed_url.'/js/mb/embed.js';
        $newURL =  $wtb_embed_url.'/js/mb/embed.js'.$auth_token;
        $content = str_replace($embedURL,$newURL,$content);
    }
    return $content;
}
add_filter( 'the_content', 'updatePageContent', 1);

/* Purpose: Function is used to append authtoken at the end of URL if user logged-in on wordpress site and custom menu is activated and used custom link for Forum in menu instead of default page.
Parameter: Manual link and post id.
Return: Manual links */
function addAuthtokenCustomLink($items, $args){
	// Append authtoken in the forum link if user logged-in on WordPress site and forum open in window independently.
	if(is_user_logged_in()) {
		$urls = array();
		$custom_anchor_links = array();
		// Get forum URL and append / at the end of the URL if not exist.
		$wtbForumURL = get_option('websitetoolbox_url');
		if(substr($wtbForumURL, -1) != '/') {
			$wtbForumURL .= "/";
		}

		foreach ($items as $key => $item) {
			$urls[$item->ID] = $item->url;
			// Check Forum URL with the custom link. If forum URl exists in the custom link then append authtoken into it.
			if(substr($item->url, -1) != '/') {
				$item->url .= "/";
			}
			if ($item->object == 'custom' && $item->url == $wtbForumURL) {
				$add_token = appendTokenInURL();
				$item->url = add_query_arg( $add_token, $item->url );
			}
		}
	}
	return $items;
}
add_filter('wp_nav_menu_objects', 'addAuthtokenCustomLink', 10, 2);

/* Purpose: create an array and set authtoken and remember-me as value.
Parameter: none
Return: array */
function appendTokenInURL () {
	$authtoken = $_COOKIE['wt_logout_token'];
	if(isset($_COOKIE['wtchat_logout_token'])) {
		$authtoken .= "-".$_COOKIE['wtchat_logout_token'];
	}
	if(isset($_COOKIE['wt_login_remember'])) {
		$remember = $_COOKIE['wt_login_remember'];
	}
	return array('authtoken' => $authtoken, 'remember' => $remember);
}

register_activation_hook( __FILE__, 'websitetoolbox_activate' );
register_deactivation_hook( __FILE__, 'websitetoolbox_deactivate' );
?>
